package cn.com.qmth.scancentral.task.thread;

import cn.com.qmth.scancentral.enums.ExamStatusCheckMode;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.StudentService;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.InputStream;

public class ExamStatusImportThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ExamStatusImportThread.class);

    private ExamStatusCheckMode mode;

    private InputStream inputStream;

    private Long examId;

    private StudentService studentService;

    private ConcurrentService concurrentService;

    public ExamStatusImportThread(Long examId, ExamStatusCheckMode mode, InputStream inputStream,
            StudentService studentService, ConcurrentService concurrentService) {
        this.examId = examId;
        this.mode = mode;
        this.inputStream = inputStream;
        this.studentService = studentService;
        this.concurrentService = concurrentService;
    }

    @Override
    public void run() {
        if (examId == null) {
            return;
        }
        if (mode == null) {
            return;
        }
        try {
            if (concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + examId).tryLock()) {
                studentService.importExamStatus(examId, mode, inputStream);
            }
        } catch (Exception e) {
            log.error("exam status import error", e);
            throw new StatusException("缺考校验导入失败");
        } finally {
            concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + examId).unlock();
        }
    }

}
