package cn.com.qmth.scancentral.task.thread;

import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.StudentService;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExamStatusResetThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ExamStatusResetThread.class);

    private Integer examNumberFillCount;

    private Long examId;

    private StudentService studentService;

    private ConcurrentService concurrentService;

    public ExamStatusResetThread(Long examId, Integer examNumberFillCount, StudentService studentService,
            ConcurrentService concurrentService) {
        this.examId = examId;
        this.examNumberFillCount = examNumberFillCount;
        this.studentService = studentService;
        this.concurrentService = concurrentService;
    }

    @Override
    public void run() {
        if (examId == null) {
            return;
        }
        if (examNumberFillCount == null) {
            return;
        }
        try {
            if (concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + examId).tryLock()) {
                studentService.resetExamStatus(examId, examNumberFillCount);
            }
        } catch (Exception e) {
            log.error("exam status reset error", e);
            throw new StatusException("缺考校验重新生成失败");
        } finally {
            concurrentService.getLock(LockType.EXAM_STATUS_RESET + "-" + examId).unlock();
        }
    }

}
